%**************************************************************************************************************************************
% Este arquivo  um apndice virtual do artigo:
% Estimao das tabelas auxiliares de impostos e margens da Matriz de Insumo-Produto com mnima perda de informao: algoritmo RAWS.
% Revista: Pesquisa e Planejamento Econmico. Braslia: Ipea, 2014.
% Autor: Thiago Sevilhano Martinez
%**************************************************************************************************************************************

%**************************************************************************************************************************************
% RAWSBASE96 - ano base 1996
% Aplicao do algoritmo RAWS ao ano 1996
% Obs: para 1996, TS no tem entradas negativas, tratar como TC e TP
%                 MC deve ser tratado como MT
%**************************************************************************************************************************************

clear
load('MIP1996');

T0 = [TC0;TP0;TS0];
T  = T0;
t  = [tc;tp;ts];
TC = TC0;
TP = TP0;
TS = TS0;

% separar entradas com sinais positivos e negativos das matrizes
[n,m] = size(H);
MC_N = zeros(n,m);
MT_N = zeros(n,m);
MC_N(MC0<0) = MC0(MC0<0);
MT_N(MT0<0) = MT0(MT0<0);
MC_P = MC0 - MC_N;
MT_P = MT0 - MT_N;
T    = [TC;TP;TS];
h_mc = (ones(1,n)*MC_N)';
h_mt = (ones(1,n)*MT_N)';

% Rodada inicial

% Na estimativa inicial das matrizes j houve o balanceamento nas linhas,
% logo deve-se assumir que os vetores r so unitrios. Optou-se por iniciar
% o balanceamento pelas colunas (vetores s). Assim, a estimativa inicial
% de W dever ser uma matriz com nmero 1 em todas as entradas.

r_tc = ones(length(tc),1); %estimativa inicial de r_tc, vetor unitrio
r_tp = ones(length(tp),1); %estimativa inicial de r_tp, vetor unitrio
r_ts = ones(length(ts),1); %estimativa inicial de r_ts, vetor unitrio
r_t  = [r_tc;r_tp;r_ts]; %estimativa inicial de r_t, vetor unitrio
r_mc = ones(length(mc),1); %estimativa inicial de r_mc, vetor unitrio
r_mt = ones(length(mt),1); %estimativa inicial de r_mt, vetor unitrio
W    = ones(n,m); %estimativa inicial de W, matriz com nmero 1 nas entradas
WT   = [W;W;W]; % equivalente s 3 submatrizes de T empilhadas

B1 = ones(1,3*n)*diag(r_t)*(T.*WT);
s_t  = h./(B1)';
s_t(h==0) = 1;
s_mc = ((ones(1,n)*(diag(r_mc)\(-MC_N./W)))'./(ones(1,n)*diag(r_mc)*(MC_P.*W))').^0.5;
s_mc(h_mc==0) = 1;
s_mt = ((ones(1,n)*(diag(r_mt)\(-MT_N./W)))'./(ones(1,n)*diag(r_mt)*(MT_P.*W))').^0.5;
s_mt(h_mt==0) = 1;

% Segundo passo

% balanceamento de W tomando dados r e s da rodada inicial
C1  = (diag(r_tc)*TC + diag(r_tp)*TP + diag(r_ts)*TS)*diag(s_t) + diag(r_mc)*MC_P*diag(s_mc) + diag(r_mt)*MT_P*diag(s_mt);
C2  = diag(r_mc)\MC_N/diag(s_mc) + diag(r_mt)\MT_N/diag(s_mt);
W   = (H+(H.^2 - 4*C1.*C2).^0.5)./(2*C1);
W_N = C2./H;
W(C1==0) = W_N(C1==0);
W(H==0) = 1;
WT = [W;W;W];

% Terceiro passo

% primeiro balanceamento completo: r, s e w 
r_tc = tc./((TC.*W)*diag(s_t)*ones(m,1));
r_tc(tc==0) = 1;
r_tp = tp./((TP.*W)*diag(s_t)*ones(m,1));
r_tp(tp==0) = 1;
r_ts = ts./((TS.*W)*diag(s_t)*ones(m,1));
r_ts(ts==0) = 1;
r_t  = [r_tc;r_tp;r_ts]; 
r_mc = mc./((MC_P.*W)*diag(s_mc)*ones(m,1));
r_mcN = (((MC_N./W)/diag(s_mc))*ones(m,1))./mc;
r_mc(mc<0) = r_mcN(mc<0);
r_mc(mc<=0) = 1;
r_mt = mt./((MT_P.*W)*diag(s_mt)*ones(m,1));
r_mtN = (((MT_N./W)/diag(s_mt))*ones(m,1))./mt;
r_mt(mt<0) = r_mtN(mt<0);
r_mt(mt==0) = 1;

B1 = ones(1,3*n)*diag(r_t)*(T.*WT);
s_t  = h./(B1)';
s_t(h==0) = 1;
s_mc = ((ones(1,n)*(diag(r_mc)\(-MC_N./W)))'./(ones(1,n)*diag(r_mc)*(MC_P.*W))').^0.5;
s_mc(h_mc==0) = 1;
s_mt = ((ones(1,n)*(diag(r_mt)\(-MT_N./W)))'./(ones(1,n)*diag(r_mt)*(MT_P.*W))').^0.5;
s_mt(h_mt==0) = 1;

C1  = (diag(r_tc)*TC + diag(r_tp)*TP + diag(r_ts)*TS)*diag(s_t) + diag(r_mc)*MC_P*diag(s_mc) + diag(r_mt)*MT_P*diag(s_mt);
C2  = diag(r_mc)\MC_N/diag(s_mc) + diag(r_mt)\MT_N/diag(s_mt);
W3 = (H+(H.^2 - 4*C1.*C2).^0.5)./(2*C1);
W_N = C2./H;
W3(C1==0) = W_N(C1==0);
W3(H==0) = 1;
 
% Iterao
DIF = W3 - W;
iter = 3                    % contagem da iterao
eps = 0.000001;             % nvel de tolerancia (equivale a R$ 1,00)
F1 = eps*(ITERM/iter);      % quando iter chega a ITERM, F1 = eps
F2 = max(max(abs(DIF)));    % retorna o maior elemento (escalar) de DIF = W3 - W
F = min([F2;F1]);           % interromper quando iter=ITERM ou maior variao em W menor que R$ 1,00   
while (F > eps)
    W = W3;
    WT = [W;W;W];
    r_tc = tc./((TC.*W)*diag(s_t)*ones(m,1));
    r_tc(tc==0) = 1;
    r_tp = tp./((TP.*W)*diag(s_t)*ones(m,1));
    r_tp(tp==0) = 1;
    r_ts = ts./((TS.*W)*diag(s_t)*ones(m,1));
    r_ts(ts==0) = 1;
    r_t  = [r_tc;r_tp;r_ts]; 
    r_mc = mc./((MC_P.*W)*diag(s_mc)*ones(m,1));
    r_mcN = (((MC_N./W)/diag(s_mc))*ones(m,1))./mc;
    r_mc(mc<0) = r_mcN(mc<0);
    r_mc(mc<=0) = 1;
    r_mt = mt./((MT_P.*W)*diag(s_mt)*ones(m,1));
    r_mtN = (((MT_N./W)/diag(s_mt))*ones(m,1))./mt;
    r_mt(mt<0) = r_mtN(mt<0);
    r_mt(mt==0) = 1;
    
    B1 = ones(1,3*n)*diag(r_t)*(T.*WT);
    s_t  = h./(B1)';
    s_t(h==0) = 1;
    s_mc = ((ones(1,n)*(diag(r_mc)\(-MC_N./W)))'./(ones(1,n)*diag(r_mc)*(MC_P.*W))').^0.5;
    s_mc(h_mc==0) = 1;
    s_mt = ((ones(1,n)*(diag(r_mt)\(-MT_N./W)))'./(ones(1,n)*diag(r_mt)*(MT_P.*W))').^0.5;
    s_mt(h_mt==0) = 1;

    C1  = (diag(r_tc)*TC + diag(r_tp)*TP + diag(r_ts)*TS)*diag(s_t) + diag(r_mc)*MC_P*diag(s_mc) + diag(r_mt)*MT_P*diag(s_mt);
    C2  = diag(r_mc)\MC_N/diag(s_mc) + diag(r_mt)\MT_N/diag(s_mt);
    W3 = (H+(H.^2 - 4*C1.*C2).^0.5)./(2*C1);
    W_N = C2./H;
    W3(C1==0) = W_N(C1==0);
    W3(H==0) = 1;

    DIF  = W3 - W;
    iter = iter + 1
    F1 = eps*(ITERM/iter);
    F2 = max(max(abs(DIF)));
    F = min([F2;F1]);
end

%ltimo passo: N+1

W = W3;

r_tc = tc./((TC.*W)*diag(s_t)*ones(m,1));
r_tc(tc==0) = 1;
r_tp = tp./((TP.*W)*diag(s_t)*ones(m,1));
r_tp(tp==0) = 1;
r_ts = ts./((TS.*W)*diag(s_t)*ones(m,1));
r_ts(ts==0) = 1;
r_t  = [r_tc;r_tp;r_ts]; 
r_mc = mc./((MC_P.*W)*diag(s_mc)*ones(m,1));
r_mcN = (((MC_N./W)/diag(s_mc))*ones(m,1))./mc;
r_mc(mc<0) = r_mcN(mc<0);
r_mc(mc<=0) = 1;
r_mt = mt./((MT_P.*W)*diag(s_mt)*ones(m,1));
r_mtN = (((MT_N./W)/diag(s_mt))*ones(m,1))./mt;
r_mt(mt<0) = r_mtN(mt<0);
r_mt(mt==0) = 1;

B1 = ones(1,3*n)*diag(r_t)*(T.*WT);
s_t  = h./(B1)';
s_t(h==0) = 1;
s_mc = ((ones(1,n)*(diag(r_mc)\(-MC_N./W)))'./(ones(1,n)*diag(r_mc)*(MC_P.*W))').^0.5;
s_mc(h_mc==0) = 1;
s_mt = ((ones(1,n)*(diag(r_mt)\(-MT_N./W)))'./(ones(1,n)*diag(r_mt)*(MT_P.*W))').^0.5;
s_mt(h_mt==0) = 1;

%Resultados

TC = diag(r_tc)*(TC.*W)*diag(s_t);
TP = diag(r_tp)*(TP.*W)*diag(s_t);
TS = diag(r_ts)*(TS.*W)*diag(s_t);
MC = diag(r_mc)*(MC_P.*W)*diag(s_mc) + diag(r_mc)\(MC_N./W)/diag(s_mc);
MT = diag(r_mt)*(MT_P.*W)*diag(s_mt) + diag(r_mt)\(MT_N./W)/diag(s_mt);
iter
F2

% Verificao de Resultados

NANrtc = sum(sum(isnan(r_tc)));
NANrtp = sum(sum(isnan(r_tp)));
NANrts = sum(sum(isnan(r_ts)));
NANrmc = sum(sum(isnan(r_mc)));
NANrmt = sum(sum(isnan(r_mt)));
INFrtc = sum(sum(isinf(r_tc)));
INFrtp = sum(sum(isinf(r_tp)));
INFrts = sum(sum(isinf(r_ts)));
INFrmc = sum(sum(isinf(r_mc)));
INFrmt = sum(sum(isinf(r_mt)));

NANst  = sum(sum(isnan(s_t)));
NANsmc = sum(sum(isnan(s_mc)));
NANsmt = sum(sum(isnan(s_mt)));
INFst  = sum(sum(isinf(s_t)));
INFsmc = sum(sum(isinf(s_mc)));
INFsmt = sum(sum(isinf(s_mt)));

NANw   = sum(sum(isnan(W)));
INFw   = sum(sum(isinf(W)));

NANtc = sum(sum(isnan(TC)));
NANtp = sum(sum(isnan(TP)));
NANts = sum(sum(isnan(TS)));
NANmc = sum(sum(isnan(MC)));
NANmt = sum(sum(isnan(MT)));

INFtc = sum(sum(isinf(TC)));
INFtp = sum(sum(isinf(TP)));
INFts = sum(sum(isinf(TS)));
INFmc = sum(sum(isinf(MC)));
INFmt = sum(sum(isinf(MT)));

Nan_Inf_r = [NANrtc NANrtp NANrts NANrmc NANrmt;INFrtc INFrtp INFrts INFrmc INFrmt]
Nan_Inf_s = [NANst NANsmc NANsmt;INFst INFsmc INFsmt]
Nan_Inf_w = [NANw; INFw]
Nan_Inf_A = [NANtc NANtp NANts NANmc NANmt;INFtc INFtp INFts INFmc INFmt]


% Passo3: Gravar no Excel
xlswrite('MIP1996', TC(1:80,1:43), 'TC', 'D6:AT85');
xlswrite('MIP1996', TC(1:80,44:47), 'TC', 'AV6:AY85');
1
xlswrite('MIP1996', TP(1:80,1:43), 'TP', 'D6:AT85');
xlswrite('MIP1996', TP(1:80,44:47), 'TP', 'AV6:AY85');
2
xlswrite('MIP1996', TS(1:80,1:43), 'TS', 'D6:AT85');
xlswrite('MIP1996', TS(1:80,44:47), 'TS', 'AV6:AY85');
3
xlswrite('MIP1996', MC(1:80,1:43), 'MC', 'D6:AT85');
xlswrite('MIP1996', MC(1:80,44:47), 'MC', 'AV6:AY85');
4
xlswrite('MIP1996', MT(1:80,1:43), 'MT', 'D6:AT85');
xlswrite('MIP1996', MT(1:80,44:47), 'MT', 'AV6:AY85');
5

save('MIP1996raws')